/**
 * 
 */
package org.springblade.datasource;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Statement;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollectionUtil;
import org.springblade.analytics.sensors.AnalyticsAgent;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.datasource.constants.DbConstant;
import org.springblade.datasource.constants.DsConstant;
import org.springblade.datasource.enums.ColumnTypeEnum;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DetailColumnTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.datasource.model.TableViewDto;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.report.entity.ReportDatasource;

import com.alibaba.druid.util.JdbcUtils;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * @author wookvn
 *
 */
@Slf4j
public abstract class AbstractGmetryDialect implements IGmetryDialect {

	protected final Map<ViewColumnTypeEnum, String> columnTypeMap = new ConcurrentHashMap<>();
	
	protected final Map<FunctionMethodEnum, String> functionMethodMap = new ConcurrentHashMap<>();
	
	protected final Map<DateFormatEnum, String> dateFormatMap = new ConcurrentHashMap<>();

	@Autowired
	protected BladeRedis bladeRedis;
	
	@Override
	public List<TableViewDto> listTableViewsByDatasource(ReportDatasource datasource) {
		return listTableViewsByDatasource(datasource, null);
	}
	
	@Override
	public List<TableColumnDto> listTableColumnsByTableViewName(ReportDatasource datasource, String tableViewName) {
		if(datasource == null){
			return null;
		}
		List<TableColumnDto> list = new ArrayList<TableColumnDto>();
		Connection connection = null;
		long startTime = System.currentTimeMillis();
		try {
			connection = DynamicDataSourceSupport.createConnection(datasource);
			DatabaseMetaData metaData = connection.getMetaData();
			String schemaPattern = "%" + dbSchema(datasource) + "%";
			ResultSet rs = metaData.getColumns(null, schemaPattern, tableViewName, "%");
			ResultSet pkRs = metaData.getPrimaryKeys(null, schemaPattern, tableViewName);
			List<String> primaryKeyColumnList = new ArrayList<>();
			while(pkRs.next()) {
				primaryKeyColumnList.add(pkRs.getString("COLUMN_NAME"));
			}
			while(rs.next()) {
				TableColumnDto columnDto = new TableColumnDto();
				columnDto.setColumnComments(rs.getString("REMARKS"));
				columnDto.setDataType(rs.getString("TYPE_NAME"));
				columnDto.setColumnName(rs.getString("COLUMN_NAME"));
				columnDto.setColumnSize(rs.getInt("COLUMN_SIZE"));
				columnDto.setDecimalDigits(rs.getInt("DECIMAL_DIGITS"));
				columnDto.setColumnType(ViewColumnTypeEnum.resolve(rs.getInt("DATA_TYPE")));
				columnDto.setDetailColumnType(DetailColumnTypeEnum.resolve(rs.getInt("DATA_TYPE")));
				//IS_NULLABLE - YES/NO  NULLABLE - 1(允许为空)/2(不确定)/0(不允许为空)
				columnDto.setNullAbleStatus("yes".equalsIgnoreCase(rs.getString("IS_NULLABLE")) ? 1 : 0); 
				//是否主键
				columnDto.setPrimaryKeyStatus(primaryKeyColumnList.contains(rs.getString("COLUMN_NAME")) ? 1 : 0);
				//IS_AUTOINCREMENT YES/NO 是否自增
				if(primaryKeyColumnList.contains(rs.getString("COLUMN_NAME")) && "YES".equals(rs.getString("IS_AUTOINCREMENT"))) {
					columnDto.setAutoIncrementStatus(1);
				}
				list.add(columnDto);
			}
			pkRs.close();
			rs.close();
			AnalyticsAgent.trackDatasource(datasource, null, "select "+tableViewName+" columns from jdbc DatabaseMetaData", System.currentTimeMillis() - startTime);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			AnalyticsAgent.trackDatasource(datasource, e, "select "+tableViewName+" columns from jdbc DatabaseMetaData", System.currentTimeMillis() - startTime);
		} finally {
			JdbcUtils.close(connection);
		}
		return list;
	}

	@Override
	public String getColumnType(ViewColumnTypeEnum columnType, Integer columnSize, Integer decimalDigits) {
		String value = columnTypeMap.get(columnType);
		if(StrUtil.isEmpty(value)) {
			throw new ServiceException("数据类型不存在|"+columnType);
		}
		if(columnType == ViewColumnTypeEnum.STRING) {
			return String.format(value, columnSize == null || columnSize <= 0 ? DbConstant.DEFAULT_COLUMN_STRING_SIZE : columnSize);
		}
		if(columnType == ViewColumnTypeEnum.NUMBER) {
			columnSize = columnSize == null || columnSize <= 0 ? DbConstant.DEFAULT_COLUMN_NUMBER_SIZE : columnSize;
			decimalDigits = decimalDigits == null ? DbConstant.DEFAULT_COLUMN_NUMBER_DIGITS : decimalDigits;
			return String.format(value, columnSize, decimalDigits);
		}
		return value;
	}
	
	@Override
	public String formatDateTime(DateFormatEnum dateTimeFormat, String columnSql) {
		String value = dateFormatMap.get(dateTimeFormat);
		if(StrUtil.isEmpty(value)) {
			throw new ServiceException("日期类型不存在|"+dateTimeFormat+"或不支持该格式转换");
		}
		return formatMethodFunction(FunctionMethodEnum.DATE_TO_STR, columnSql, value);
	}
	
	@Override
	public String formatMethodFunction(FunctionMethodEnum functionMethod, Object... args) {
		String value = functionMethodMap.get(functionMethod);
		if(StrUtil.isEmpty(value)) {
			throw new ServiceException("函数方法不存在|"+functionMethod+"或不支持该函数方法");
		}
		return String.format(value, args);
	}
	
	@Override
	public String buildCreateViewSql(String viewName, String sqlStatement) {
		return String.format(CubeConstants.CREATE_REPLACE_VIEW_SQL_FORMAT, viewName, sqlStatement);
	}
	
	@Override
	public String convertDataType(ViewColumnTypeEnum oldColumnType, ViewColumnTypeEnum newColumnType, String columnSql,
			DateFormatEnum dateFormat) {
		FunctionMethodEnum functionMethod = null;
		if(oldColumnType == ViewColumnTypeEnum.STRING && newColumnType == ViewColumnTypeEnum.DATETIME) {
			functionMethod = FunctionMethodEnum.STR_TO_DATE;
		}else if(oldColumnType == ViewColumnTypeEnum.NUMBER && newColumnType == ViewColumnTypeEnum.DATETIME) {
			functionMethod = FunctionMethodEnum.NUMBER_TO_DATE;
		}else if(oldColumnType == ViewColumnTypeEnum.DATETIME && newColumnType == ViewColumnTypeEnum.STRING) {
			functionMethod = FunctionMethodEnum.DATE_TO_STR;
		}
		if(oldColumnType == null || oldColumnType == newColumnType || functionMethod == null) {
			return columnSql;
		}
		String dateTimeFormat = "";
		if(dateFormat != null && (oldColumnType == ViewColumnTypeEnum.DATETIME || newColumnType == ViewColumnTypeEnum.DATETIME)) {
			dateTimeFormat = dateFormatMap.get(dateFormat);
			if(StrUtil.isEmpty(dateTimeFormat)) {
				throw new ServiceException("日期类型不存在|"+dateTimeFormat+"或不支持日期格式");
			}
		}
		return formatMethodFunction(functionMethod, columnSql, dateTimeFormat);
	}

	@Override
	public List<TableViewDto> listTableViewsByDatasource(ReportDatasource datasource,String tableName) {
		if(datasource == null){
			return null;
		}
		//1. 优先从缓存取数据
		List<TableViewDto> cacheTableViewDtoList = listTableViewsFromCache(datasource.getId());
		if(CollectionUtil.isNotEmpty(cacheTableViewDtoList)) {
			return StrUtil.isBlank(tableName) ? cacheTableViewDtoList : cacheTableViewDtoList.stream().filter(item -> item.getTableViewName().contains(tableName)).collect(Collectors.toList());
		}

		//2. 缓存没数据，从数据源查询
		List<TableViewDto> list = new ArrayList<>();
		Connection connection = null;
		long startTime = System.currentTimeMillis();
		try {
			connection = DynamicDataSourceSupport.createConnection(datasource);
			DatabaseMetaData metaData = connection.getMetaData();
			String schemaPattern = "%" + dbSchema(datasource) + "%",
					tableNamePattern = StrUtil.isNotBlank(tableName) ? ("%" + tableName + "%") : null;
			ResultSet rs = metaData.getTables(null, schemaPattern, tableNamePattern, new String[] {"TABLE", "VIEW", "REMOTE TABLE", "MEMORY TABLE"});
			while(rs.next()) {
				list.add(new TableViewDto(rs.getString("TABLE_NAME"), rs.getString("REMARKS"), datasource.getId()));
			}
			rs.close();

			//3.设置缓存数据
			listTableViewsSetCache(datasource.getId(), list);

			AnalyticsAgent.trackDatasource(datasource, null, "select tables from jdbc DatabaseMetaData", System.currentTimeMillis() - startTime);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			AnalyticsAgent.trackDatasource(datasource, e, "select tables from jdbc DatabaseMetaData", System.currentTimeMillis() - startTime);
		} finally {
			JdbcUtils.close(connection);
		}
		return list;
	}
	
	@Override
	public void executeSql(List<String> sqls, ReportDatasource datasource) throws Exception {
		Connection connection = null;
        Statement statement = null;
        try {
            connection = DynamicDataSourceSupport.createConnection(datasource);
            statement = connection.createStatement();
            connection.setAutoCommit(false);
            for (String sql : sqls) {
				long startTime = System.currentTimeMillis();
                statement.execute(sql);
				AnalyticsAgent.trackDatasource(datasource, null, sql, System.currentTimeMillis() - startTime);
            }
            connection.commit();
        } catch(Exception e){
            if(connection != null) {
				connection.rollback();
			}
            AnalyticsAgent.trackDatasource(datasource, e, "unknown", 0L);
            throw e;
        } finally {
            JdbcUtils.close(statement);
            JdbcUtils.close(connection);
        }
	}

	protected List<TableViewDto> listTableViewsFromCache(Long dsId) {
		Set cacheSet = null;
		try { //缓存失败，不影响主业务
			String key = DsConstant.CACHE_KEY_PREFIX + dsId;
			cacheSet = bladeRedis.sMembers(key);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		List<TableViewDto> dataList = null;
		if(CollectionUtil.isNotEmpty(cacheSet)) {
			dataList = new ArrayList<>(cacheSet);
		}
		return dataList;
	}

	protected void listTableViewsSetCache(Long dsId, List<TableViewDto> list) {
		try { //缓存失败，不影响主业务
			String key = DsConstant.CACHE_KEY_PREFIX + dsId;
			if(CollectionUtil.isNotEmpty(list)) {
				bladeRedis.sAdd(key, list.stream().toArray(TableViewDto[]::new));
				bladeRedis.expire(key, Duration.ofMinutes(DsConstant.CACHE_TIME_OUT_MIN));
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
	}
}
