/**
 *
 */
package org.springblade.datasource.dialect;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.bson.BsonDocument;
import org.bson.BsonValue;
import org.bson.Document;
import org.springblade.analytics.sensors.AnalyticsAgent;
import org.springblade.core.tool.utils.Func;
import org.springblade.cube.model.FileStructure;
import org.springblade.datasource.AbstractGmetryDialect;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.datasource.model.TableViewDto;
import org.springblade.report.entity.ReportDatasource;
import org.springframework.stereotype.Component;

import com.mongodb.ConnectionString;
import com.mongodb.MongoClientSettings;
import com.mongodb.client.ListCollectionsIterable;
import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoCursor;
import com.mongodb.client.MongoDatabase;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * <p>mongodb的实现体</p>
 * @author  wookvn
 * @datetime 2024-10-29 17:19:46
 */
@Slf4j
@Component
public class MongoGmetryDialect extends AbstractGmetryDialect {

	@Override
	public List<String> buildCreateTableSql(String tableName, String tableComment, List<FileStructure> structureList) {
		return null;
	}

	@Override
	public List<TableColumnDto> listTableColumnsByTableViewName(ReportDatasource datasource, String tableViewName) {
		if(datasource == null || StrUtil.isBlank(tableViewName)){
			return null;
		}
		List<TableColumnDto> columnList = new ArrayList<>();
		long startTime = System.currentTimeMillis();
		try (MongoClient mongoClient = _createConnection(datasource)) {
			MongoDatabase mongoClientDatabase = mongoClient.getDatabase(datasource.getDsDb());
			MongoCollection<BsonDocument> collection = mongoClientDatabase.getCollection(tableViewName, BsonDocument.class);
			Document query = new Document();
			for(BsonDocument doc : collection.find(query).limit(1)) {
				for (Map.Entry<String, BsonValue> valueEntry : doc.entrySet()) {
					TableColumnDto tableColumnDto = new TableColumnDto();
					tableColumnDto.setColumnName(valueEntry.getKey());
					tableColumnDto.setDataType(valueEntry.getValue().getBsonType().name());
					columnList.add(tableColumnDto);
				}
			}
			AnalyticsAgent.trackDatasource(datasource, null, "select collections from mongoDatasource", System.currentTimeMillis() - startTime);
		}catch (Exception e){
			log.error(e.getMessage(), e);
		}
		return columnList;
	}

	@Override
	public List<TableViewDto> listTableViewsByDatasource(ReportDatasource datasource, String tableName) {
		if(datasource == null){
			return null;
		}
		//1. 优先从缓存取数据
		List<TableViewDto> cacheTableViewDtoList = listTableViewsFromCache(datasource.getId());
		if(CollectionUtil.isNotEmpty(cacheTableViewDtoList)) {
			return StrUtil.isBlank(tableName) ? cacheTableViewDtoList : cacheTableViewDtoList.stream().filter(item -> item.getTableViewName().contains(tableName)).collect(Collectors.toList());
		}
		//2. 查询db
		// 创建MongoClient实例
		long startTime = System.currentTimeMillis();
		try (MongoClient mongoClient = _createConnection(datasource)) {
			MongoDatabase mongoClientDatabase = mongoClient.getDatabase(datasource.getDsDb());
			ListCollectionsIterable<Document> documents = mongoClientDatabase.listCollections();
			List<TableViewDto> dataList = new ArrayList<>();
			if(documents != null) {
				for(MongoCursor<Document> it = documents.iterator(); it.hasNext();) {
					Document document = it.next();
					TableViewDto tableView = new TableViewDto();
					tableView.setDatasourceId(datasource.getId());
					tableView.setTableViewName(document.getString("name"));
					dataList.add(tableView);
				}
				//3.设置缓存数据
				listTableViewsSetCache(datasource.getId(), dataList);
				// 埋点
				AnalyticsAgent.trackDatasource(datasource, null, "select Documents from collection["+tableName+"]", System.currentTimeMillis() - startTime);
			}
			return dataList;
		} catch (Exception e) {
			AnalyticsAgent.trackDatasource(datasource, e, "select Documents from collection["+tableName+"]", System.currentTimeMillis() - startTime);
			log.error(e.getMessage(), e);
			return null;
		}
	}

	@Override
	public DsTypeEnum dsType() {
		return DsTypeEnum.mongo;
	}

	/**
	 * <p>创建连接</p>
	 * @author wookvn
	 * @datetime 2024-10-14 15:06:20
	 * @param ds: 数据源对象
	 * @return com.mongodb.client.MongoClient
	 **/
	private static MongoClient _createConnection(ReportDatasource ds) {
		// 创建连接字符串
		ConnectionString connectionString = new ConnectionString(ds.getUrl());
		int connectTimeout = Func.toInt(DsTypeEnum.mongo.jdbcConnectionPropTimeout().get("connectTimeout")),
				readTimeout = Func.toInt(DsTypeEnum.mongo.jdbcConnectionPropTimeout().get("socketTimeout"));

		// 创建MongoClientSettings.Builder
		MongoClientSettings.Builder settingsBuilder = MongoClientSettings.builder()
				.applyConnectionString(connectionString)
				// 连接超时时间，单位毫秒
				.applyToSocketSettings(builder -> builder.connectTimeout(connectTimeout, TimeUnit.MILLISECONDS))
				// 套接字读取超时时间，单位毫秒
				.applyToSocketSettings(builder -> builder.readTimeout(readTimeout, TimeUnit.MILLISECONDS));

		//构建MongoClientSettings
		MongoClientSettings settings = settingsBuilder.build();
		return MongoClients.create(settings);
	}

	/**
	 * <p>校验连接是否正常</p>
	 * @author wookvn
	 * @datetime 2024-10-14 14:43:37
	 * @param ds: 数据源对象
	 * @return java.lang.String
	 **/
	public static String checkConnection(ReportDatasource ds) {
		// 创建MongoClient实例
		try (MongoClient mongoClient = _createConnection(ds)) {
			mongoClient.listDatabaseNames().first();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return e.getMessage();
		}
		return null;
	}
}
