/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Query;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.datasource.constants.DsConstant;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.datasource.model.TableViewDto;
import org.springblade.report.dto.DelTableParams;
import org.springblade.report.dto.ReportViewDTO;
import org.springblade.report.dto.response.TableCategoryDTO;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.entity.TableCategory;
import org.springblade.report.entity.TableCategoryRelation;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.report.service.ITableCategoryRelationService;
import org.springblade.report.service.ITableCategoryService;
import org.springblade.report.service.normal.ReportDatasourcePermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 产品报表基础信息 控制器
 *
 * @author BladeX
 * @since 2021-03-10
 */
@RestController
@RequestMapping("report/baseInfo")
@Api(value = "数据库表基础信息", tags = "数据库表基础信息 ReportBaseController")
public class ReportBaseController extends BladeController {

    @Resource
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private DialectContext dialectContext;

    @Autowired
    private ReportDatasourcePermissionService reportDatasourcePermissionService;

    @Autowired
    private ITableCategoryRelationService tableCategoryRelationService;

    @Autowired
    private ITableCategoryService tableCategoryService;

    @Autowired
    private DynamicTableService dynamicTableService;
    @Autowired
    private CubeDataService cubeDataService;
    @Autowired
    private BladeRedis bladeRedis;

    /**
     * 获取报表视图
     */
    @GetMapping("/listViews")
    @Deprecated
    public R<?> list() {
        return R.data(null);
    }

    /**
     * 获取报表视图
     */
    @GetMapping("/listViewsByDataSource")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "根据数据源获取报表视图", notes = "")
    public R<IPage<TableCategoryDTO>> list(
            @ApiParam("数据源Id") @RequestParam Long datasourceId,
            @ApiParam("分类id") @RequestParam(required = false) Long classifyId,
            @ApiParam("表名") @RequestParam(required = false) String tableName,
            @ApiParam("分页参数") Query query) throws ClassNotFoundException, SQLException {
        return R.data(getTableView(datasourceId, classifyId, tableName, query, getUser()));
    }

    /**
     * 获取报表视图的字段信息
     */
    @GetMapping("/listColumnsByDatasource")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "根据数据源获取报表视图的字段信息", notes = "")
    public R<List<TableColumnDto>> listColumnsByDatasource(ReportViewDTO reportViewDTO) {
        String viewName = reportViewDTO.getTableViewName();
        if (StrUtil.isAllBlank(viewName)) {
            return R.fail("视图名称不可以为空");
        }
        ReportDatasource datasource = reportDatasourceService.getById(reportViewDTO.getDatasourceId());
        if (null == datasource) {
            return R.fail("数据源id错误");
        }
        List<TableColumnDto> list = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableColumnsByTableViewName(datasource, viewName);
        return R.data(list);
    }

    public IPage<TableCategoryDTO> getTableView(Long datasourceId, Long classifyId, String tableName, Query query, BladeUser user) {
        ReportDatasource datasource = reportDatasourceService.getById(datasourceId);
        if (null == datasource) {
            throw new ServiceException("数据源不存在，请检查");
        }
        List<TableViewDto> list = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableViewsByDatasource(datasource);

        if (CollectionUtil.isEmpty(list)) {
            return new Page<>();
        }

        List<TableViewDto> dataList = new ArrayList<>();
        if (StrUtil.isNotEmpty(tableName)) {
            dataList.addAll(list.stream().filter(tableViewDto -> tableViewDto.getTableViewName().contains(tableName) || (tableViewDto.getTableComments() != null && tableViewDto.getTableComments().contains(tableName))).collect(Collectors.toList()));
        } else {
            dataList.addAll(list);
        }

        Long currUserId = getUser().getUserId();
        //数据源不是当前登录人，处理权限组逻辑
        if (!datasource.getCreateUserId().equals(currUserId)&& !AuthUtil.isAdmin()) {
            reportDatasourcePermissionService.dealTablePermission(currUserId, datasourceId, permTableNameSet -> dataList.removeIf(it -> !permTableNameSet.contains(it.getTableViewName())));
        }

        if (CollectionUtil.isEmpty(dataList)) {
            return new Page<>();
        }

        IPage<TableCategoryDTO> page = new Page<>();
        page.setSize(Optional.ofNullable(query.getSize()).orElse(20));
        page.setCurrent(Optional.ofNullable(query.getCurrent()).orElse(1));

        if ((classifyId == null || classifyId < 1)) {
            int totalSize = dataList.size();
            page.setTotal(totalSize);
            if (totalSize >= page.getCurrent() * page.getSize()) {
                List<TableCategoryDTO> subList = dataList.stream().sorted(Comparator.comparing(TableViewDto::getTableViewName)).skip((page.getCurrent() - 1) * page.getSize())
                        .limit(page.getSize()).map(item -> BeanUtil.toBean(item, TableCategoryDTO.class)).collect(Collectors.toList());
                this.handleCategory(subList);
                page.setRecords(subList);
            } else {
                List<TableCategoryDTO> subList = dataList.stream().sorted(Comparator.comparing(TableViewDto::getTableViewName))
                        .skip((page.getCurrent() - 1) * page.getSize())
                        .map(item -> BeanUtil.toBean(item, TableCategoryDTO.class)).collect(Collectors.toList());
                this.handleCategory(subList);
                page.setRecords(subList);
            }
        } else {
            LambdaQueryWrapper<TableCategoryRelation> lambdaQueryWrapper = Wrappers.<TableCategoryRelation>lambdaQuery()
                    .eq(TableCategoryRelation::getCategoryId, classifyId)
                    .like(StrUtil.isNotEmpty(tableName), TableCategoryRelation::getTableViewName, tableName)
                    .orderByDesc(TableCategoryRelation::getOrderIndex);
            Page<TableCategoryRelation> tableCategoryRelations = tableCategoryRelationService.page(new Page<>(page.getCurrent(), page.getSize()), lambdaQueryWrapper);

            //过滤需要权限访问的表
            TableCategory tableCategory = tableCategoryService.getById(classifyId);

            List<TableCategoryDTO> data = new ArrayList<>();
            if (CollectionUtil.isNotEmpty(tableCategoryRelations.getRecords())) {
                tableCategoryRelations.getRecords().forEach(tableCategoryRelation -> {
                    TableCategoryDTO dto = new TableCategoryDTO();
                    if (null != tableCategory) {
                        dto.setCategoryName(tableCategory.getCategoryName());
                        dto.setDatasourceId(tableCategory.getDatasourceId());
                    }
                    dto.setCategoryId(tableCategoryRelation.getCategoryId());
                    dto.setOrderIndex(tableCategoryRelation.getOrderIndex());
                    dto.setTableViewName(tableCategoryRelation.getTableViewName());
                    dto.setTableComments(tableCategoryRelation.getTableComments());
                    data.add(dto);
                });
            }
            page.setTotal(tableCategoryRelations.getTotal());
            page.setRecords(data);
            page.setPages(tableCategoryRelations.getPages());
        }
        return page;
    }


    /**
     * 补充表视图补充添加分类信息
     *
     * @param data
     * @return void
     */
    public void handleCategory(List<TableCategoryDTO> data) {
        if (CollectionUtil.isEmpty(data)) {
            return;
        }
        List<String> tableViewNameList = data.stream().map(TableCategoryDTO::getTableViewName).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(tableViewNameList)) {
            return;
        }
        LambdaQueryWrapper<TableCategoryRelation> lambdaQueryWrapper = Wrappers.<TableCategoryRelation>lambdaQuery()
                .in(TableCategoryRelation::getTableViewName, tableViewNameList);
        List<TableCategoryRelation> categoryRelationList = tableCategoryRelationService.list(lambdaQueryWrapper);
        if (CollectionUtil.isEmpty(categoryRelationList)) {
            return;
        }
        List<Long> categoryIdList = categoryRelationList.stream().map(TableCategoryRelation::getCategoryId).collect(Collectors.toList());
        LambdaQueryWrapper<TableCategory> queryWrapper = Wrappers.<TableCategory>lambdaQuery()
                .in(TableCategory::getId, categoryIdList);
        List<TableCategory> categoryList = tableCategoryService.list(queryWrapper);
        if (CollectionUtil.isEmpty(categoryList)) {
            return;
        }
        Map<String, TableCategoryRelation> relationMap = categoryRelationList.stream().collect(Collectors.toMap(TableCategoryRelation::getTableViewName, Function.identity(), (t1, t2) -> t1));
        Map<Long, TableCategory> categoryMap = categoryList.stream().collect(Collectors.toMap(TableCategory::getId, Function.identity(), (t1, t2) -> t1));
        data.forEach(dto -> {
            if (relationMap.containsKey(dto.getTableViewName())) {
                dto.setCategoryId(relationMap.get(dto.getTableViewName()).getCategoryId());
                dto.setTableComments(relationMap.get(dto.getTableViewName()).getTableComments());
                if (categoryMap.containsKey(dto.getCategoryId())) {
                    dto.setCategoryName(categoryMap.get(dto.getCategoryId()).getCategoryName());
                    dto.setDatasourceId(categoryMap.get(dto.getCategoryId()).getDatasourceId());
                }
            }
        });
    }

    /**
     * 删除表
     */
    @PostMapping("/delTable")
    public R<Boolean> delTable(@RequestBody DelTableParams delTableParams) {
        ReportDatasource datasource = reportDatasourceService.getById(delTableParams.getDsId());
        if (null == datasource) {
            return R.fail("数据源id错误");
        }
        //检查是否有数据集使用
        Long count = cubeDataService.lambdaQuery()
                .eq(CubeData::getDatasourceId, datasource.getId())
                .like(CubeData::getCubeTableNames, delTableParams.getTableName())
                .count();
        if (count > 0) {
            return R.fail("该表有数据集在使用，请先删除数据集");
        }
        dynamicTableService.executeSql(datasource, String.format("DROP TABLE %s;", delTableParams.getTableName()));
        //删除缓存
        String key = DsConstant.CACHE_KEY_PREFIX + datasource.getId();
        bladeRedis.del(key);
        return R.success("删除成功");
    }
}
