package org.springblade.visual.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.lang.hash.Hash;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.analytics.sensors.AnalyticsAgent;
import org.springblade.config.properties.AutoReviewProperties;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.visual.dto.*;
import org.springblade.visual.entity.VisualComponentTemplateCategory;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataTemplate;
import org.springblade.visual.enums.VisualDataTemplateCategory;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualDataTemplateCategoryMapper;
import org.springblade.visual.mapper.VisualDataTemplateMapper;
import org.springblade.visual.utils.YxApiUtils;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.*;

import static io.protostuff.MapSchema.MessageFactories.HashMap;

@Service
public class VisualDataTemplateService extends ServiceImpl<VisualDataTemplateMapper, VisualDataTemplate> {

    @Autowired
    private VisualDataTemplateCategoryMapper visualDataTemplateCategoryMapper;
    @Autowired
    @Lazy
    private VisualDataTemplateManageService visualDataTemplateManageService;
    @Autowired
    private VisualReviewLogService visualReviewLogService;
    @Autowired
    private VisualDataService visualDataService;
    @Autowired
    private AutoReviewProperties autoReviewProperties;
    @Value("${blade.pdfUrl}")
    private String pdfUrl;
    @Autowired
    private YxApiUtils yxApiUtils;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;


    public List<VisualDataTemplateCategory> categoryList(String name) {
        List<VisualDataTemplateCategory> categoryList = visualDataTemplateCategoryMapper
                .selectList(Wrappers.<VisualDataTemplateCategory>lambdaQuery()
                        .like(StringUtils.isNotBlank(name), VisualDataTemplateCategory::getName, name));
        return categoryList;
    }

    public boolean submitCategory(VisualDataTemplateCategory category) {
        if (category.getId() == null) {
            category.setCreateUser(SecureUtil.getUserId());
            visualDataTemplateCategoryMapper.insert(category);
        } else {
            category.setUpdateUser(SecureUtil.getUserId());
            visualDataTemplateCategoryMapper.updateById(category);
        }
        return true;
    }

    public boolean deleteCategory(Long id) {
        //判断是否有子分类
        List<VisualDataTemplateCategory> list = visualDataTemplateCategoryMapper.selectList(Wrappers.<VisualDataTemplateCategory>lambdaQuery().eq(VisualDataTemplateCategory::getParentId, id));
        if (list.size() > 0) {
            throw new ServiceException("请先删除子分类");
        }
        //判断目录下面是否有模板
        List<VisualDataTemplate> templateList = this.lambdaQuery().eq(VisualDataTemplate::getCategory, id).list();
        if (CollectionUtil.isNotEmpty(templateList)) {
            throw new ServiceException("目录下存在模板");
        }
        visualDataTemplateCategoryMapper.deleteById(id);
        return true;
    }


    public Page<VisualDataTemplate> listTemplates(ListTemplateParams params) {
        LambdaUpdateWrapper<VisualDataTemplate> wrapper = Wrappers.<VisualDataTemplate>lambdaUpdate()
                .eq(params.getCategory() != null, VisualDataTemplate::getCategory, params.getCategory())
                .like(params.getName() != null, VisualDataTemplate::getName, params.getName())
                .eq(params.getCreateUser() != null, VisualDataTemplate::getCreateUser, params.getCreateUser())
                .eq(params.getReviewStatus() != null, VisualDataTemplate::getReviewStatus, params.getReviewStatus())
                .eq(params.getStatus() != null, VisualDataTemplate::getStatus, params.getStatus())
                .between(params.getStartTime() != null && params.getEndTime() != null, VisualDataTemplate::getCreateTime, params.getStartTime(), params.getEndTime())
                .orderByDesc(VisualDataTemplate::getCreateTime);
        Page<VisualDataTemplate> page = this.page(new Page<>(params.getPage(), params.getSize()), wrapper);
        return page;
    }

    public Boolean removeTemplate(Long id) {
        return this.removeById(id);
    }

    public Boolean checkOperationPermissions(Long id) {
        VisualDataTemplate dataTemplate = this.getById(id);
        //等于0代表只有预览权限 校验是否是创建人
        if (dataTemplate.getOperationPermissions() == 0) {
            if (dataTemplate.getCreateUser() != AuthUtil.getUserId()) {
                throw new ServiceException("该模板只有查看权限");
            }
        }
        return true;
    }


    @Transactional(rollbackFor = Exception.class)
    public Boolean copyTemplate(Long id, BladeUser bladeUser) {
        VisualDataTemplate dataTemplate = this.getById(id);
        //等于0代表只有预览权限 校验是否是创建人
        if (dataTemplate.getOperationPermissions() == 0) {
            if (dataTemplate.getCreateUser() != AuthUtil.getUserId()) {
                throw new ServiceException("该模板只有查看权限");
            }
        }
        dataTemplate.setName("[副本]" + dataTemplate.getName()).setId(null);
        List<Long> visualIdList = visualDataService
                .doCopy(CollectionUtil.newHashSet(dataTemplate.getVisualId()),
                        null, null, null,
                        VisualTypeEnum.TEMPLATE, bladeUser, null, null);
        dataTemplate.setVisualId(visualIdList.get(0))
                .setStatus(0)
                .setCreateUser(AuthUtil.getUserId())
                .setCreateUserName(AuthUtil.getNickName())
                .setReviewStatus(0);
        dataTemplate.setTenantId(AuthUtil.getTenantId());
        this.save(dataTemplate);
        return true;
    }

    public List<VisualReviewLog> listReviewLogs(Long id) {
        return visualReviewLogService.getBaseMapper().listByRelationIdAndType(id, 0);
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean submitReview(@RequestBody SubmitTemplateReviewParams params) {
        this.lambdaUpdate().eq(VisualDataTemplate::getId, params.getTemplateId())
                .set(VisualDataTemplate::getOperationPermissions, params.getOperationPermissions())
                .update();
        VisualReviewLog visualReviewLog = new VisualReviewLog();
        visualReviewLog.setRemark(params.getReason())
                .setStatus(0)
                .setType(0)
                .setRelationId(params.getTemplateId())
                .setCreateTime(LocalDateTime.now())
                .setCreateUserName(AuthUtil.getNickName())
                .setCreateUserId(SecureUtil.getUserId());
        visualReviewLogService.save(visualReviewLog);
        //修改审核状态
        this.lambdaUpdate()
                .eq(VisualDataTemplate::getId, params.getTemplateId())
                .set(VisualDataTemplate::getReviewStatus, 1)
                .update();
        //自动审核
        if (autoReviewProperties.getVisualSwitch() && StrUtil.isNotBlank(params.getUrl())) {
            taskExecutor.execute(() -> {
                templateAutoReview(params.getUrl(), params.getTemplateId(), params.getToken());
            });
        }
        return true;
    }

    /**
     * 自动审核
     */
    @Transactional(rollbackFor = Exception.class)
    public void templateAutoReview(String url, Long templateId, String token) {
        VisualDataTemplate dataTemplate = this.getById(templateId);
        if (dataTemplate != null) {
            VisualData visualData = visualDataService.getById(dataTemplate.getVisualId());
            //组装pdf接口参数
            String content = visualData.getContent();
            Map<String, Object> param = new HashMap<>();
            Map<String, Object> site = new HashMap<>();
            site.put("content", JSONUtil.parseObj(content));
            site.put("id", templateId);
            site.put("url", url);
            param.put("site", Arrays.asList(site));
            param.put("token", token);
            param.put("watermark", 0);
            String resBody = HttpUtil.post(pdfUrl, JSON.toJSONString(param), 300000);
            R r = JSON.parseObject(resBody, R.class);
            if (r.getCode() == 200) {
                List<String> urls = JSON.parseArray(r.getData().toString(), String.class);
                if (CollectionUtil.isNotEmpty(urls)) {
                    YxConclusionCheckParams yxConclusionCheckParams = new YxConclusionCheckParams();
                    yxConclusionCheckParams
                            .setAppId(autoReviewProperties.getAppid())
                            .setType("IMAGE");
                    YxConclusionCheckParams.DataItem dataItem = new YxConclusionCheckParams.DataItem();
                    dataItem.setContent(urls.get(0))
                            .setDataId(UUID.fastUUID().toString());
                    yxConclusionCheckParams.setDataList(Arrays.asList(dataItem));
                    //研祥自动审核接口
                    YxConclusionCheckDto yxConclusionCheckDto = yxApiUtils.conclusionCheck(yxConclusionCheckParams);
                    //根据结果修改审核状态
                    this.getBaseMapper().updateTemplate(
                            yxConclusionCheckDto.getSuggestion() == 0 ? 1 : 0,
                            yxConclusionCheckDto.getSuggestion() == 0 ? 2 : 3,
                            templateId
                    );
                    VisualReviewLog visualReviewLog = new VisualReviewLog();
                    visualReviewLog.setTenantId(dataTemplate.getTenantId());
                    visualReviewLog
                            .setType(0)
                            .setRelationId(templateId)
                            .setRemark("自动审核" + (StrUtil.isNotBlank(yxConclusionCheckDto.getLabel())?": "+yxConclusionCheckDto.getLabel():""))
                            .setStatus(yxConclusionCheckDto.getSuggestion() == 0 ? 1 : 2)
                            .setCreateUserName(dataTemplate.getCreateUserName())
                            .setCreateTime(LocalDateTime.now())
                            .setCreateUserId(dataTemplate.getCreateUser());
                    visualReviewLogService.save(visualReviewLog);
                }
            }
        }

    }

    /**
     * 修改大屏模板基本信息
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateTemplateBasicInfo(VisualDataTemplate template) {
        this.updateById(template);
        visualDataService.lambdaUpdate()
                .eq(VisualData::getId, template.getVisualId())
                .set(VisualData::getCategory, template.getCategory())
                .update();
        return true;
    }

    /**
     * 所有审核通过的模板
     */
    public Page<VisualDataTemplate> allReviewedTemplate(Integer page, Integer size, String name, String category) {
        return this.getBaseMapper().allReviewedTemplate(new Page(page, size), name, category);
    }

    /**
     * 所有目录
     */
    public List<VisualDataTemplateCategory> allMcpCategory(String name) {
        List<VisualDataTemplateCategory> categoryList = visualDataTemplateCategoryMapper.allMcpCategory(name);
        return Optional.ofNullable(categoryList).orElse(new ArrayList<>());
    }

    /**
     * 大屏模版计数
     *
     * @param templateId
     */
    @Async
    public void addTemplateUseTimes(Long templateId) {
        baseMapper.addUseTimes(templateId);
    }

    /**
     * 大屏模版埋点
     *
     * @param visualDataId
     * @param templateSource
     * @param templateId
     */
    @Async
    public void track(Long visualDataId, Integer templateSource, Long templateId) {
        VisualDataTemplate visualDataTemplate = this.getById(templateId);
        VisualDataTemplateCategory visualDataTemplateCategory = visualDataTemplateCategoryMapper.selectById(visualDataTemplate.getCategory());
        String templateCateName = visualDataTemplateCategory != null ? visualDataTemplateCategory.getName() : "";
        AnalyticsAgent.trackVisualTemplate(visualDataId, templateSource, visualDataTemplate, templateCateName);
    }
}
