/**
 *
 */
package org.springblade.visual.service;

import java.time.LocalDateTime;
import java.util.*;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.json.JSONUtil;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.utils.Func;
import org.springblade.visual.dto.VisualPowerInfoDto;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataPermission;
import org.springblade.visual.entity.VisualPowerPointInfo;
import org.springblade.visual.entity.VisualPowerPointPermission;
import org.springblade.visual.enums.PowerPinotTypeEnum;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualPowerPointInfoMapper;
import org.springblade.visual.mapper.VisualPowerPointPermissionMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;

/**
 *	幻灯片 服务处理
 *
 * @author wookvn
 * @date 2023-02-04
 */
@Slf4j
@Service
public class VisualPowerPointInfoService extends ServiceImpl<VisualPowerPointInfoMapper, VisualPowerPointInfo> {

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private VisualPowerPointPermissionService visualPowerPointPermissionService;

    /**
     *
     * @param ppidList
     * @return
     */
    public List<VisualPowerPointInfo> listByPpidGroup(List<Long> ppidList) {
        return baseMapper.listByPpidGroup(ppidList);
    }

    /**
     * 根据ID删除一个幻灯片
     * @param id
     * @return
     */
    @Transactional
    public boolean doRemove(Long id) {
        LambdaQueryWrapper<VisualData> queryWrapper = Wrappers.<VisualData>lambdaQuery().select(VisualData::getId).eq(VisualData::getPowerPointId, id).eq(VisualData::getVisualType, VisualTypeEnum.POWER_POINT);
        List<Long> visualIdList = visualDataService.listObjs(queryWrapper, a -> Long.valueOf(a.toString()));
        if (CollectionUtil.isNotEmpty(visualIdList)) {  //删除大屏
            visualDataService.doRemove(visualIdList);
        }
        // 删除与幻灯片绑定的权限组
        visualPowerPointPermissionService.remove(Wrappers.<VisualPowerPointPermission>lambdaQuery()
                .eq(VisualPowerPointPermission::getPpId, id));
        return baseMapper.deleteById(id) > 0;
    }

    /**
     * 添加幻灯片的大屏列表
     * @param ppId
     * @param visualIds
     * @return
     */
    @Transactional
    public List<Long> doAddVisual(Long ppId, String visualIds, BladeUser bladeUser) {
        Set<Long> visualIdSet = new HashSet<>(Func.toLongList(visualIds));
        List<Long> newVisualIdList = null;
        if (CollectionUtil.isNotEmpty(visualIdSet)) { //根据原大屏ID复制一份新的大屏
            newVisualIdList = visualDataService.doCopyPowerPoint(visualIdSet, ppId, bladeUser);
            //更新时间
            VisualPowerPointInfo entity = new VisualPowerPointInfo();
            entity.setId(ppId);
            entity.setUpdateBy(bladeUser.getNickName());
            entity.setUpdateTime(LocalDateTime.now());
            baseMapper.updateById(entity);
        }
        return newVisualIdList;
    }

    private void setVisualCount(Long ppid) {
        Long count = visualDataService.lambdaQuery().eq(VisualData::getPowerPointId, ppid).count();
        this.lambdaUpdate().eq(VisualPowerPointInfo::getId, ppid).set(VisualPowerPointInfo::getVisualCount, count).update();
    }

    /**
     * 删除单张大屏
     * @param ppId
     * @param visualId
     * @return
     */
    @Transactional
    public boolean doRemoveVisual(Long ppId, Long visualId, BladeUser bladeUser) {
        boolean ret = visualDataService.remove(Wrappers.<VisualData>lambdaQuery().eq(VisualData::getVisualType, VisualTypeEnum.POWER_POINT).eq(VisualData::getPowerPointId, ppId).eq(VisualData::getId, visualId));
        if (ret) {
            VisualPowerPointInfo entity = new VisualPowerPointInfo();
            entity.setId(ppId);
            entity.setUpdateBy(bladeUser.getNickName());
            entity.setUpdateTime(LocalDateTime.now());
            baseMapper.updateById(entity);
        }
        setVisualCount(ppId);
        return ret;
    }

    /**
     * 添加或者更新幻灯片
     * @param visualPowerInfoDto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean saveOrUpdatePower(VisualPowerInfoDto visualPowerInfoDto, BladeUser bladeUser) {

        VisualPowerPointInfo visualPowerPointInfo = BeanUtil.copyProperties(visualPowerInfoDto, VisualPowerPointInfo.class);
        //如果是幻灯片 统计数量
        Long count = visualDataService.lambdaQuery().eq(VisualData::getPowerPointId, visualPowerInfoDto.getId()).count();
        visualPowerPointInfo.setVisualCount(count.intValue());
        boolean saveOrUpdate = this.saveOrUpdate(visualPowerPointInfo);
        // 如果不是普通的幻灯片那么就可能是我的收藏和幻灯片归档，此时需要复制一份
        if (visualPowerInfoDto.getVisualType() != null &&
                visualPowerInfoDto.getVisualType() != PowerPinotTypeEnum.GENERAL.getValue()) {
            if (CollectionUtil.isNotEmpty(visualPowerPointInfo.getVisualIds())) {
                Set<Long> visualIdSet = new HashSet<>();
                for (String visualId : visualPowerInfoDto.getVisualIds()) {
                    visualIdSet.add(Long.valueOf(visualId));
                }
                visualDataService.doCopyPowerPoint(visualIdSet, visualPowerPointInfo.getId(), bladeUser);
            }
        }
        // 为幻灯片设置权限组信息
        if (saveOrUpdate && visualPowerInfoDto.getId() == null
                && CollectionUtil.isNotEmpty(visualPowerInfoDto.getDataPermIdList())) {
            List<VisualPowerPointPermission> dataList = new ArrayList<>(visualPowerInfoDto.getDataPermIdList().size());
            visualPowerInfoDto.getDataPermIdList().forEach(t -> {
                VisualPowerPointPermission entity = new VisualPowerPointPermission();
                entity.setCreateBy(bladeUser.getNickName());
                entity.setCreateTime(LocalDateTime.now());
                entity.setCreateUserId(bladeUser.getUserId());
                entity.setDataPermId(t);
                entity.setPpId(visualPowerPointInfo.getId());
                dataList.add(entity);
            });
            return visualPowerPointPermissionService.saveBatch(dataList);
        }

        return saveOrUpdate;
    }

    /**
     * 批量删除大屏
     * @param ppId
     * @param visualIds
     * @param user
     * @return
     */
    @Transactional
    public boolean doRemoveVisualList(Long ppId, String visualIds, BladeUser user) {
        List<Long> visualIdList = new ArrayList<>();
        for (String s : visualIds.split(",")) {
            visualIdList.add(Long.valueOf(s));
        }
        boolean ret = visualDataService.remove(Wrappers.<VisualData>lambdaQuery()
                .eq(VisualData::getVisualType, VisualTypeEnum.POWER_POINT)
                .eq(VisualData::getPowerPointId, ppId)
                .in(VisualData::getId, visualIdList));
        if (ret) {
            VisualPowerPointInfo entity = new VisualPowerPointInfo();
            entity.setId(ppId);
            entity.setUpdateBy(user.getNickName());
            entity.setUpdateTime(LocalDateTime.now());
            baseMapper.updateById(entity);
        }
        return ret;
    }
}